package com.edufound.bytedance.activity;

import android.animation.ObjectAnimator;
import android.app.Activity;
import android.content.Intent;
import android.graphics.Color;
import android.os.Build;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.view.animation.LinearInterpolator;
import android.webkit.WebView;
import android.widget.FrameLayout;
import android.widget.ImageView;

import androidx.annotation.Nullable;

import com.edufound.bytedance.R;
import com.edufound.bytedance.base.BaseActivity;
import com.edufound.bytedance.mvp.model.MainModel;
import com.edufound.bytedance.mvp.presenter.MainPersenter;
import com.edufound.bytedance.mvp.view.MainView;
import com.edufound.bytedance.util.ContextUtil;

public class MainActivity extends BaseActivity<MainModel, MainView, MainPersenter> implements MainView {

    FrameLayout mMainFrame;
    Activity mActivity;
    WebView mWebView;
    ImageView mLoading;
    ObjectAnimator mLoadingAnim;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        mActivity = this;
        init();
    }


    void init() {
        mMainFrame = findViewById(R.id.MainFrame);
        mLoading = findViewById(R.id.MainLoading);
        mWebView = new WebView(ContextUtil.getmApplicationContext());
        mMainFrame.setBackgroundColor(Color.parseColor("#5CC9FF"));
        mWebView.setBackgroundColor(Color.parseColor("#5CC9FF"));
        mLoadingAnim = ObjectAnimator.ofFloat(mLoading, "rotation", 0.0F, 359.0F);//设置Y轴的立体旋转动画
        mLoadingAnim.setRepeatCount(-1);
        mLoadingAnim.setDuration(1000);
        LinearInterpolator interpolator = new LinearInterpolator();
        mLoadingAnim.setInterpolator(interpolator); //设置匀速旋转，不卡顿 icon_anim.start();
        //設置webview
        mPresenter.initWeb();
        mPresenter.openWeb(getIntent());
        mMainFrame.addView(getWebView());
//        /test show network dialog
//        Button ben = new Button(mActivity);
//        ben.setLayoutParams(new FrameLayout.LayoutParams(100, 100));
//        ben.setText("click show dialog error page");
//        ben.setOnClickListener(new View.OnClickListener() {
//            @Override
//            public void onClick(View view) {
//                Intent intent = new Intent(MainActivity.this, DialogNetworkActivity.class);
//                startActivity(intent);
//            }
//        });
//        mMainFrame.addView(ben);
    }


    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    public void onPause() {
        super.onPause();
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        return mPresenter.onKeyDown(keyCode, event);
    }

    @Override
    public MainView createView() {
        return this;
    }

    @Override
    public MainModel createModel() {
        return new MainModel();
    }

    @Override
    public MainPersenter createPresenter() {
        return new MainPersenter();
    }


    @Override
    public Activity getActivity() {
        return mActivity;
    }

    @Override
    public void showProgressDialog() {
        mLoading.setVisibility(View.VISIBLE);
        mLoading.bringToFront();
        mLoadingAnim.start();
    }

    @Override
    public void dismissProgressDialog() {
        mLoading.setVisibility(View.INVISIBLE);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            mLoadingAnim.pause();
        }
    }

    @Override
    public void loadJsMehtod(String method) {
        mWebView.loadUrl("javascript:" + method + "");
    }

    @Override
    public void loadJsMethodPost(final String method) {
        mWebView.post(new Runnable() {
            @Override
            public void run() {
                mWebView.loadUrl("javascript:" + method + "");
            }
        });
    }

    @Override
    public void loadUrl(String url) {
        mWebView.loadUrl(url);
    }

    @Override
    public WebView getWebView() {
        return mWebView;
    }

    @Override
    public void showDialogNetWork() {
        Intent intent = new Intent(mActivity, DialogNetworkActivity.class);
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        mActivity.startActivity(intent);
    }
}
