package com.edufound.bytedance.application;

import android.app.Activity;
import android.app.Application;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Process;

import com.edufound.bytedance.receiver.HomeKeyEventReceiver;
import com.edufound.bytedance.receiver.NetworkChangeReceiver;
import com.edufound.bytedance.util.ContextUtil;
import com.edufound.bytedance.util.Logger;
import com.edufound.bytedance.util.OkHttpUtil;
import com.umeng.analytics.MobclickAgent;
import com.umeng.commonsdk.UMConfigure;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import okhttp3.Request;

public class MyApplication extends Application {
    //网络异常监听
    NetworkChangeReceiver mNetworkReceiver;
    //按home键的receiver
    HomeKeyEventReceiver mHomeReceiver;

    private int activityAount = 0;
    public boolean isForeground = false;
    //语文预习
    private static String UMENG_KEY_YUWEN_PREVIEW = "5fbcd71b1e29ca3d7be369a9";
    private static String APP_CODE_YUWEN_PREVIEW = "4001";
    //语文复习
    private static String UMENG_KEY_YUWEN_REVIEW = "5fbcd7731e29ca3d7be369be";
    private static String APP_CODE_YUWEN_REVIEW = "4002";

    //数学预习
    private static String UMENG_KEY_MATHEMATICS_PREVIEW = "";
    private static String APP_CODE_MATHEMATICS_PREVIEW = "4003";
    //数学复习
    private static String UMENG_KEY_MATHEMATICS_REVIEW = "";
    private static String APP_CODE_MATHEMATICS_REVIEW = "4004";

    @Override
    public void onCreate() {
        super.onCreate();
        ContextUtil.setAppStartTime(System.currentTimeMillis());
        ContextUtil.setmApplicationContext(this);
        initUMeng();
        initBoardCastReceiver();
    }


    void initBoardCastReceiver() {
        mHomeReceiver = new HomeKeyEventReceiver();
        registerReceiver(mHomeReceiver, new IntentFilter(Intent.ACTION_CLOSE_SYSTEM_DIALOGS));
        mNetworkReceiver = new NetworkChangeReceiver();
        IntentFilter netFilter = new IntentFilter();
        netFilter.addAction("android.net.conn.CONNECTIVITY_CHANGE");
        registerReceiver(mNetworkReceiver, netFilter);
        registerActivityLifecycleCallbacks(activityLifecycleCallbacks);
    }


    public static String getAppCode() {
        if (ContextUtil.getmApplicationContext().getPackageName().contains("yuwen.prepare")) {
            //语文预习
            return APP_CODE_YUWEN_PREVIEW;
        } else if (ContextUtil.getmApplicationContext().getPackageName().contains("yuwen.review")) {
            //语文复习
            return APP_CODE_YUWEN_REVIEW;
        } else if (ContextUtil.getmApplicationContext().getPackageName().contains("mathematics.prepare")) {
            //数学复习
            return APP_CODE_MATHEMATICS_PREVIEW;
        } else if (ContextUtil.getmApplicationContext().getPackageName().contains("mathematics.review")) {
            //数学复习
            return APP_CODE_MATHEMATICS_REVIEW;
        }
        return "default";
    }

    void initUMeng() {
        try {
            String umeng_key = "";
            int app_code = Integer.valueOf(getAppCode());
            switch (app_code) {
                case 4001:
                    umeng_key = UMENG_KEY_YUWEN_PREVIEW;
                    break;
                case 4002:
                    umeng_key = UMENG_KEY_YUWEN_REVIEW;
                    break;
                case 4003:
                    umeng_key = UMENG_KEY_MATHEMATICS_PREVIEW;
                    break;
                case 4004:
                    umeng_key = UMENG_KEY_MATHEMATICS_REVIEW;
                    break;
            }
            // 初始化SDK
            UMConfigure.init(ContextUtil.getmApplicationContext(), umeng_key, String.valueOf(app_code), UMConfigure.DEVICE_TYPE_PHONE, null);
            // 选用AUTO页面采集模式
            MobclickAgent.setPageCollectionMode(MobclickAgent.PageMode.AUTO);
        } catch (Exception e) {
            Logger.e("友盟初始化失败:" + e.getMessage());
            e.printStackTrace();
        }
    }

    private static List<Activity> getAllActivitys() {
        List<Activity> list = new ArrayList<>();
        try {
            Class<?> activityThread = Class.forName("android.app.ActivityThread");
            Method currentActivityThread = activityThread.getDeclaredMethod("currentActivityThread");
            currentActivityThread.setAccessible(true);
            //获取主线程对象
            Object activityThreadObject = currentActivityThread.invoke(null);
            Field mActivitiesField = activityThread.getDeclaredField("mActivities");
            mActivitiesField.setAccessible(true);
            Map<Object, Object> mActivities = (Map<Object, Object>) mActivitiesField.get(activityThreadObject);
            for (Map.Entry<Object, Object> entry : mActivities.entrySet()) {
                Object value = entry.getValue();
                Class<?> activityClientRecordClass = value.getClass();
                Field activityField = activityClientRecordClass.getDeclaredField("activity");
                activityField.setAccessible(true);
                Object o = activityField.get(value);
                list.add((Activity) o);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return list;
    }

    public static void exitApp() {
        List<Activity> myAclist = getAllActivitys();
        for (int i = 0; i < myAclist.size(); i++) {
            myAclist.get(i).finish();
        }
        android.os.Process.killProcess(Process.myPid());
    }

    /**
     * Activity 生命周期监听，用于监控app前后台状态切换
     */
    ActivityLifecycleCallbacks activityLifecycleCallbacks = new ActivityLifecycleCallbacks() {
        @Override
        public void onActivityCreated(Activity activity, Bundle savedInstanceState) {
        }

        @Override
        public void onActivityStarted(Activity activity) {
            if (activityAount == 0) {
                //app回到前台
                isForeground = true;
//                ToastUtil.showToast("app回到前台了");
            }
            activityAount++;
        }

        @Override
        public void onActivityResumed(Activity activity) {
        }

        @Override
        public void onActivityPaused(Activity activity) {
        }

        @Override
        public void onActivityStopped(Activity activity) {
            activityAount--;
            if (activityAount == 0) {
                isForeground = false;
//                ToastUtil.showToast("app后台了");
                OkHttpUtil.postUserTime(new OkHttpUtil.ResultCallback() {
                    @Override
                    public void onError(Request request, Exception e) {
                        Logger.e("上传使用时间：onError:" + request.toString());
                        exitApp();
                        android.os.Process.killProcess(android.os.Process.myPid());
                    }

                    @Override
                    public void onResponse(Object response) {
                        Logger.e("上传使用时间：" + response.toString());
                        exitApp();
                        android.os.Process.killProcess(android.os.Process.myPid());
                    }
                });

            }
        }

        @Override
        public void onActivitySaveInstanceState(Activity activity, Bundle outState) {
        }

        @Override
        public void onActivityDestroyed(Activity activity) {
        }
    };
}
