package com.edufound.bytedance.util;

import android.app.Activity;
import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.telephony.TelephonyManager;
import android.util.DisplayMetrics;
import android.util.TypedValue;

import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.util.Locale;

public class DeviceUtil {

    private static DisplayMetrics metric = new DisplayMetrics();

    /**
     * @return 序列号 SerialNumber
     * @author zhangmengjie
     */
    public String SerialNumber() {

        return android.os.Build.SERIAL;
    }


    /**
     * 获取宽度
     */
    public static int getWidth(Activity context) {
        context.getWindowManager().getDefaultDisplay().getMetrics(metric);
        return metric.widthPixels; // 屏幕宽度（像素）
    }

    /**
     * 获取宽度
     */
    public static int getHeight(Activity context) {
        context.getWindowManager().getDefaultDisplay().getMetrics(metric);
        return metric.heightPixels; // 屏幕高度（像素）
    }

    public float getDensity(Activity context) {
        context.getWindowManager().getDefaultDisplay().getMetrics(metric);
        return metric.density; // 屏幕密度（0.75 / 1.0 / 1.5）
    }

    public int getDensityDpi(Activity context) {
        context.getWindowManager().getDefaultDisplay().getMetrics(metric);
        return metric.densityDpi; // // 屏幕密度DPI（120 / 160 / 240）
    }

    // md5加密
    public String md5Encode(String inStr) {
        MessageDigest md5 = null;
        try {
            md5 = MessageDigest.getInstance("MD5");
        } catch (Exception e) {
            System.out.println(e.toString());
            e.printStackTrace();
            return "";
        }

        byte[] byteArray = null;
        try {
            byteArray = inStr.getBytes("UTF-8");
        } catch (UnsupportedEncodingException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        byte[] md5Bytes = md5.digest(byteArray);
        StringBuffer hexValue = new StringBuffer();
        for (int i = 0; i < md5Bytes.length; i++) {
            int val = ((int) md5Bytes[i]) & 0xff;
            if (val < 16) {
                hexValue.append("0");
            }
            hexValue.append(Integer.toHexString(val));
        }
        return hexValue.toString();
    }

    /**
     * @return VersionCode
     * @author zhangmengjie
     */
    public String getVersionCode(Context context) {
        PackageManager packageManager = context.getPackageManager();
        PackageInfo packageInfo;
        String versionCode = "";
        try {
            packageInfo = packageManager.getPackageInfo(context.getPackageName(), 0);
            versionCode = packageInfo.versionCode + "";
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
        return versionCode;
    }

    /**
     * @return VersionName
     * @author zhangmengjie
     */
    public String getVersionName(Context context) {
        PackageManager packageManager = context.getPackageManager();
        PackageInfo packageInfo;
        String versionName = "";
        try {
            packageInfo = packageManager.getPackageInfo(context.getPackageName(), 0);
            versionName = packageInfo.versionName;
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
        return versionName;
    }

    /**
     * 获取当前手机系统语言。
     *
     * @return 返回当前系统语言。例如：当前设置的是“中文-中国”，则返回“zh-CN”
     */
    public String getSystemLanguage() {
        return Locale.getDefault().getLanguage();
    }

    /**
     * 获取当前系统上的语言列表(Locale列表)
     *
     * @return 语言列表
     */
    public Locale[] getSystemLanguageList() {
        return Locale.getAvailableLocales();
    }

    /**
     * 获取当前手机系统版本号
     *
     * @return 系统版本号
     */
    public String getSystemVersion() {
        return android.os.Build.VERSION.RELEASE;
    }

    /**
     * 获取手机型号
     *
     * @return 手机型号
     */
    public String getSystemModel() {
        return android.os.Build.MODEL;
    }

    /**
     * 获取手机厂商
     *
     * @return 手机厂商
     */
    public String getDeviceBrand() {
        return android.os.Build.BRAND;
    }

    /**
     * 获取手机IMEI(需要“android.permission.READ_PHONE_STATE”权限)
     *
     * @return 手机IMEI
     */
    public String getIMEI(Context ctx) {
        TelephonyManager tm = (TelephonyManager) ctx.getSystemService(Activity.TELEPHONY_SERVICE);
        if (tm != null) {
            return tm.getDeviceId();
        }
        return null;
    }

    //dp转px
    public static int dip2px(Context context, float dpValue) {
        final float scale = context.getResources().getDisplayMetrics().density;
        return (int) (dpValue * scale + 0.5f);
    }

    //px转dp
    public static int px2dip(Context context, int pxValue) {
        return ((int) TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, pxValue, context.getResources().getDisplayMetrics()));
    }

    public static void initDesignSize(Context context, int screenWidth, int screenHeight) {
        int designWidth = 960;
        int designHeight = 600;
        float ds = 1.0f * designWidth / designHeight;
        float ts = 1.0f * screenWidth / screenHeight;
        float designScale = 0.0f;
        if (ds >= ts) {
            int targetWidth = screenWidth;
            designScale = 1.0f * targetWidth / designWidth;

        } else {
            int targetHeight = screenHeight;
            designScale = 1.0f * targetHeight / designHeight;

        }
        setDisplay(context, designScale);
    }

    /**
     * 设置dpi缩放比
     */
    private static void setDisplay(Context context, float designScale) {
        DisplayMetrics displayMetrics = context.getResources().getDisplayMetrics();
        displayMetrics.density = designScale;
        displayMetrics.scaledDensity = displayMetrics.density;
        displayMetrics.xdpi = displayMetrics.density * 160;
        displayMetrics.ydpi = displayMetrics.xdpi;
        displayMetrics.densityDpi = Float.valueOf(displayMetrics.xdpi).intValue();
    }
}
